%% CSTR-distillation process generatedata_optScriptx2.m %%
% 07.06.14 
%%
% Script for optimization of reactor, separator and recycle process.
% The numerical description of the process is taken from Larsson et al (2003)
% 
% clc
% clear all
global p;
global u;
%ns=50;
ny = 10 ;
T=@(x)80-x*20;
load x_opt_cstrdest.mat
load sensFmat.mat
 
%Boiling temperature of the light component
Tb_L=353;
%Boiling temperature of the heavy component
Tb_H=373; 


% Column parameters
p.qF    = 1;
p.NT    = 22;
p.NF    = 13;
p.alpha = 2;
p.Vmax  = 1500/60;
% CSTR parameters
p.F0 = 460/60*1.0;
p.zF0 = 0.9;
p.k1 = 0.341/60; 
p.F_int = 958/60 ; 
% Flags
p.OPTI=0;
p.case_I=1;

% Initial values for disturbance and inputs:
F_int   = p.F0;
x_d_int = 0.8151 ;
L_int   = 12.97 ; 

u.u1 = x_d_int ;
u.u2 = L_int ; 

if p.case_I==1
    % Constraints
    lb=zeros(p.NT+8,1);
    ub=[ones(p.NT,1); ones(8,1)*Inf];
    
    % xB <= 0.0105
    ub(1)=0.0105;
    lb(1)=0.0105;
    % Mr <= 2800;
    ub(p.NT+7)=2800;
    lb(p.NT+7)=2800;
    % F0 = fixed; 
    lb(p.NT+8)=p.F0;
    ub(p.NT+8)=p.F0;
else
    % Constraints
    lb=zeros(p.NT+8,1);
    ub=[ones(p.NT,1); ones(8,1)*Inf];
    
    % xB <= 0.0105
    ub(1)=0.0105;
    % Mr <= 2800;
    ub(p.NT+7)=2800;
    % V <= Vmax; 
    ub(p.NT+2)=p.Vmax;
end

% Initial value
x0= x_opt;  

% Controlling active constriants: 
    % xB <= 0.0105
    ub(1)=0.0105;
    lb(1)=0.0105;
    % Mr <= 2800;
    ub(p.NT+7)=2800;
    lb(p.NT+7)=2800;

for i=1:ns
% Change in the Feed - disturbance
F0dist = 0.1;
d_F0 = (F_int-(F0dist*F_int))+(2*(F0dist*F_int).*rand(1,ns))-F_int;     
%d_F0 = d_F0(i) ; 

% Change in input L
d_u2 = (L_int-(0.1*L_int))+(2*(0.1*L_int).*rand(1,ns))-L_int;
%d_u2 = d_u2(i) ;

p.F0 = F_int + d_F0(:,i); 
u.u2 = L_int + d_u2(:,i);

% fmincon options
options = optimset('TolFun',10e-6,'TolCon',10e-6,'MaxFunEvals',1e4,...
    'Display','none','Algorithm','active-set','Diagnostics','off'...
    );

% Using boundry conditions to keep a veriable constant
% Feed:
    lb(p.NT+8)  = p.F0;
    ub(p.NT+8)  = p.F0;

% L
   lb(p.NT+1)  = u.u2;
   ub(p.NT+1)  = u.u2;


% fmincon
[x(:,i),fval(:,i),exitflag(:,i)]=fmincon(@fun,x0,[],[],[],[],lb,ub,@nlcon,options);
cost(:,i)= x(p.NT+2,i); %*60 ; 

dF0(i,:) = p.F0 ; 


Te= Tb_L*x(1:22,:)+(1-x(1:22,:))*Tb_H;
Topt= Tb_L*x_opt(1:22,:)+(1-x_opt(1:22,:))*Tb_H;

end

x_optT  = [Topt ; x_opt(23:30)];
out      = [Te(1:22,1:i) ; x(23:30,1:i)];


% In deviation variabels
dx = out(:,1:size(x,2))-kron(x_opt,ones(1,size(x,2)));
d_cost=cost-(x_opt(p.NT+2)); %*60);

exitflag ;
converge = 1./exitflag;

%% Picking the desired measurements for dx: 
xval = [dx(8,:) ; dx(13,:) ; dx(18,:) ; dx(22,:)];
y    = [dx(8,:) ; dx(13,:) ; dx(18,:) ; dx(22,:) ; dx(23:28,:) ] ;
y5   = [dx(8,:) ; dx(22,:) ; dx(23,:) ; dx(26,:) ; dx(30,:)] ;
y5nd   = [dx(8,:) ; dx(22,:) ; dx(23,:) ; dx(26,:) ; dx(27,:)] ;
y9   = [dx(8,:) ; dx(13,:) ; dx(22,:) ; dx(23:28,:) ; dx(30,:) ] ;

y_opt = [x_opt(8,:) ; x_opt(13,:) ; x_opt(18,:) ; x_opt(22,:) ; x_opt(23:28,:) ] ;
y_optT = [x_optT(8,:) ; x_optT(13,:) ; x_optT(18,:) ; x_optT(22,:) ; x_optT(23:28,:) ] ;
y_optT9 = [x_optT(8,:) ; x_optT(13,:) ; x_optT(22,:) ; x_optT(23:28,:);  x_optT(30,:) ] ;
y_optT5 = [x_optT(8,:) ; x_optT(22,:) ;x_optT(23,:) ; x_optT(26,:) ; x_optT(30,:)] ;
y_optT5nd = [x_optT(8,:) ; x_optT(22,:) ;x_optT(23,:) ; x_optT(26,:) ; x_optT(27,:)] ;
%% Adding noise to the measurements 
% Measurements: 
%y = [8      13 	  18 	22  	 Lt	   Vb      D      B 	 F      zf    ] 
Wn = [1       0       0      0       0      0      0      0      0      0   ;
      0       1       0      0       0      0      0      0      0      0   ;
      0       0       1      0       0      0      0      0      0      0   ;
      0       0       0      1       0      0      0      0      0      0   ;
      0       0       0      0       0.13   0      0      0      0      0   ;
      0       0       0      0       0      0.213  0      0      0      0   ;
      0       0       0      0       0      0      0.083  0      0      0   ;
      0       0       0      0       0      0      0      0.077  0      0   ;
      0       0       0      0       0      0      0      0      0.16   0   ;
      0       0       0      0       0      0      0      0      0      0.02];
 
%y9 = [8      13       22    Lt     Vb      D      B      F      zf    F0      ] 
Wn9 = [1      0        0    0        0      0      0      0       0     0    ;
       0      1        0    0        0      0      0      0       0     0    ;
       0      0        1    0        0      0      0      0       0     0    ;
       0      0        0    0.13     0      0      0      0       0     0    ;
       0      0        0    0        0.213  0      0      0       0     0    ;
       0      0        0    0        0      0.083  0      0       0     0    ;
       0      0        0    0        0      0      0.077  0       0     0    ;
       0      0        0    0        0      0      0      0.16    0     0    ;
       0      0        0    0        0      0      0      0       0.02  0    ;
       0      0        0    0        0      0      0      0       0     0.38 ];
   
%y = [8      22   Lt     B      F0 ] 
Wn5 = [1     0    0       0      0    ;
       0     1    0       0      0    ;
       0     0    0.13    0      0    ;
       0     0    0       0.077  0    ;
       0     0    0       0      0.38 ];  
   
%y =    [8      22   Lt     B      F ] 
Wn5nd = [1     0    0       0      0    ;
         0     1    0       0      0    ;
         0     0    0.13    0      0    ;
         0     0    0       0.077  0    ;
         0     0    0       0      0.16 ];  
       
a=kron(diag(Wn),ones(1,ns));
n = (a-(0.1*a))+(2*(0.1*a).*rand(ny,ns));
yn = y+n;

a9=kron(diag(Wn9),ones(1,ns));
n9 = (a9-(0.1*a9))+(2*(0.1*a9).*rand(ny,ns));
yn9 = y9 + n9 ; 

a5=kron(diag(Wn5),ones(1,ns));
n5 = (a5-(0.1*a5))+(2*(0.1*a5).*rand(5,ns));
yn5 = y5 + n5 ; 

a5nd=kron(diag(Wn5nd),ones(1,ns));
n5nd = (a5nd-(0.1*a5nd))+(2*(0.1*a5nd).*rand(5,ns));
yn5nd = y5nd + n5nd ; 

Wd = [0.1] ; 

Yexl=[F*Wd Wn] ; 
Yexl9=[F9*Wd Wn9] ; 
Yexl5=[F5*Wd Wn5] ; 
Yexl5nd=[F5nd*Wd Wn5nd] ; 


save('optimalval','y_optT', 'y_optT9', 'y_optT5', 'y_optT5nd')

save('cstrdest','y','d_cost','ns','y_opt','Wn','yn','Wd','Yexl','Yexl9','Yexl5','Yexl5nd','F','F9',...
'F5','F5nd','dF0','dx','y9','yn9','y5','yn5','y5nd', 'yn5nd')

% Results
if p.case_I==1
    casename='case I: min operation cost(energy)\n';
else
    casename='case II: max production rate\n';
end
%Print the results
% results_fmincon=sprintf(strcat(...
%     casename,...
%     'feed rate,             F0[kmol/h]      = %1$0.1f\n',...
%     'reactor effluent,      F[kmol/h]       = %2$0.1f\n',...
%     'vapor boilup,          V[kmol/h]       = %3$0.1f\n',...
%     'reflux,                L[kmol/h]       = %4$0.1f\n',...
%     'recycle (distilate),   D[kmol/h]       = %5$0.1f\n',...
%     'recycle composition,   xD[molA/mol]    = %6$0.4f\n',...
%     'bottom composition,    xB[molA/mol]    = %7$0.4f\n',...
%     'reactor composition,   zF[molA/mol]    = %8$0.4f\n',...
%     'reactor holdup,        Mr[kmol/h]      = %9$0.0f\n',...
%     'Column temperatures,   T_i[C]          = T_3   T_8   T_13  T_18\n ',...
%     '                                         %10$0.1f  %11$0.1f  %12$0.1f  %13$0.1f\n'...
%     ),x(p.NT+8)*60,x(p.NT+5)*60,x(p.NT+2)*60,x(p.NT+1)*60, x(p.NT+3)*60,...
%     x(p.NT), x(1), x(p.NT+6),x(p.NT+7),T(x(3)),T(x(8)),T(x(13)),T(x(18)))

run alldata_dataH_CSTRdest2.m 