% Script for optimization of reactor, separator and recycle process.
% The numerical description of the process is taken from Larsson et al (2003)

clc
clear all
global p;

T=@(x)80-x*20;

% Column parameters
p.qF    = 1;
p.NT    = 22;
p.NF    = 13;
p.alpha = 2;
p.Vmax  = 1500/60;
% CSTR parameters
p.F0 = 460/60*1.0;
p.zF0 = 0.9;
p.k1 = 0.341/60; 
% Flags
p.OPTI=0;
p.case_I=1;

if p.case_I==1
    % Constraints
    lb=zeros(p.NT+8,1);
    ub=[ones(p.NT,1); ones(8,1)*Inf];
    
    % xB <= 0.0105
    ub(1)=0.0105;
    % Mr <= 2800;
    ub(p.NT+7)=2800;
    % F0 = fixed; 
    lb(p.NT+8)=p.F0;
    ub(p.NT+8)=p.F0;
else
    % Constraints
    lb=zeros(p.NT+8,1);
    ub=[ones(p.NT,1); ones(8,1)*Inf];
    
    % xB <= 0.0105
    ub(1)=0.0105;
    % Mr <= 2800;
    ub(p.NT+7)=2800;
    % V <= Vmax; 
    ub(p.NT+2)=p.Vmax;
end

% Initial value
x0= [ones(1,p.NT)*0.5 10 15 5 5 1.1 0.5 1000 400/60]';

% fmincon options
options = optimset('TolFun',10e-6,'TolCon',10e-6,'MaxFunEvals',1e4,...
'Display','none','Algorithm','sqp','Diagnostics','off'...
);
% fmincon
[x_opt,fval,exitflag]=fmincon(@fun,x0,[],[],[],[],lb,ub,@nlcon,options);

% Results
if p.case_I==1
    casename='case I: min operation cost(energy)\n';
else
    casename='case II: max production rate\n';
end
%Print the results
results_fmincon=sprintf(strcat(...
    casename,...
    'feed rate,             F0[kmol/h]      = %1$0.1f\n',...
    'reactor effluent,      F[kmol/h]       = %2$0.1f\n',...
    'vapor boilup,          V[kmol/h]       = %3$0.1f\n',...
    'reflux,                L[kmol/h]       = %4$0.1f\n',...
    'recycle (distilate),   D[kmol/h]       = %5$0.1f\n',...
    'recycle composition,   xD[molA/mol]    = %6$0.4f\n',...
    'bottom composition,    xB[molA/mol]    = %7$0.4f\n',...
    'reactor composition,   zF[molA/mol]    = %8$0.4f\n',...
    'reactor holdup,        Mr[kmol/h]      = %9$0.0f\n',...
    'Column temperatures,   T_i[C]          = T_3   T_8   T_13  T_18\n ',...
    '                                         %10$0.1f  %11$0.1f  %12$0.1f  %13$0.1f\n'...
    ),x_opt(p.NT+8)*60,x_opt(p.NT+5)*60,x_opt(p.NT+2)*60,x_opt(p.NT+1)*60, x_opt(p.NT+3)*60,...
    x_opt(p.NT), x_opt(1), x_opt(p.NT+6),x_opt(p.NT+7),T(x_opt(3)),T(x_opt(8)),T(x_opt(13)),T(x_opt(18)))

save('x_opt_cstrdest','x_opt')