%% CSTR-distillation process simulationtest2for5ynd.m %%
% 07.06.14 
%%
% Script for optimization of reactor, separator and recycle process.
% The numerical description of the process is taken from Larsson et al (2003)
run generatedata_optScriptx2.m 
% clc
% clear all
global p;

T=@(x)80-x*20;
%Boiling temperature of the light component
Tb =353;
%Boiling temperature of the heavy component
Th=373; 

% Column parameters
p.qF    = 1;
p.NT    = 22;
p.NF    = 13;
p.alpha = 2;
p.Vmax  = 1500/60;
% CSTR parameters
p.F0 = 460/60*1.0;
p.zF0 = 0.9;
p.k1 = 0.341/60; 
% Flags
p.OPTI=0;
p.case_I=1;

if p.case_I==1
    % Constraints
    lb=zeros(p.NT+8,1);
    ub=[ones(p.NT,1); ones(8,1)*Inf];
    
    % xB <= 0.0105
    ub(1)=0.0105;
    % Mr <= 2800;
    ub(p.NT+7)=2800;
    % F0 = fixed; 
    lb(p.NT+8)=p.F0;
    ub(p.NT+8)=p.F0;
else
    % Constraints
    lb=zeros(p.NT+8,1);
    ub=[ones(p.NT,1); ones(8,1)*Inf];
    
    % xB <= 0.0105
    ub(1)=0.0105;
    % Mr <= 2800;
    ub(p.NT+7)=2800;
    % V <= Vmax; 
    ub(p.NT+2)=p.Vmax;
end

% Initial value
x0= [ones(1,p.NT)*0.5 10 15 5 5 1.1 0.5 1000 400/60]';

% fmincon options
options = optimset('TolFun',10e-6,'TolCon',10e-6,'MaxFunEvals',1e4,...
'Display','none','Algorithm','sqp','Diagnostics','off'...
);
% fmincon
[x,fval,exitflag]=fmincon(@fun1,x0,[],[],[],[],lb,ub,@nlcon1,options);

% Results
if p.case_I==1
    casename='case I: min operation cost(energy)\n';
else
    casename='case II: max production rate\n';
end
%Print the results
% results_fmincon=sprintf(strcat(...
%     casename,...
%     'feed rate,             F0[kmol/h]      = %1$0.1f\n',...
%     'reactor effluent,      F[kmol/h]       = %2$0.1f\n',...
%     'vapor boilup,          V[kmol/h]       = %3$0.1f\n',...
%     'reflux,                L[kmol/h]       = %4$0.1f\n',...
%     'recycle (distilate),   D[kmol/h]       = %5$0.1f\n',...
%     'recycle composition,   xD[molA/mol]    = %6$0.4f\n',...
%     'bottom composition,    xB[molA/mol]    = %7$0.4f\n',...
%     'reactor composition,   zF[molA/mol]    = %8$0.4f\n',...
%     'reactor holdup,        Mr[kmol/h]      = %9$0.0f\n',...
%     'Column temperatures,   T_i[C]          = T_3   T_8   T_13  T_18\n ',...
%     '                                         %10$0.1f  %11$0.1f  %12$0.1f  %13$0.1f\n'...
%     ),x(p.NT+8)*60,x(p.NT+5)*60,x(p.NT+2)*60,x(p.NT+1)*60, x(p.NT+3)*60,...
%     x(p.NT), x(1), x(p.NT+6),x(p.NT+7),T(x(3)),T(x(8)),T(x(13)),T(x(18)))

%% Controlling the process

y = [x(8) ; x(13) ; x(18) ; x(22) ; x(23:28) ] ;
F0_int = x(p.NT+8);

%Controlling only L -Brute force
% cs = x(p.NT+1) ; 
% Aeq(1,23) = 1 ;
% Aeq(1,30) = 0 ;  
% beq = cs ;

% Controlling x_D and L combinded soc
% cs = x(p.NT) - 0.0077*x(p.NT+1);
% Aeq(1,22) = 1 ;
% Aeq(1,23) = 0.0077 ;
% Aeq(1,30) = 0 ;   
% beq = cs ;

%%
% Controlling with a full H-matrix : 
 load hmatrix.mat     % loads Hmat, Hns and Hexl   
 load optimalval.mat  % loads y_opt with temperatures 



if Method == 1
    Htype = Hmat;
elseif Method == 2
    Htype = Hexl ;
else 
    Htype = [Hns(1,:)+Hns(2,:)+Hns(3,:)] ;  
end 

 %% Data method 
% Hmat is calculated with temperatur-data multiplied with optimal values of
% y with optimal temperatures 
csT = Htype*y_optT5;
 
H1 = [Htype(1,1:2)] ; % Is to be transformed into composition
H2 = [Htype(1,3:5)]; % Is to be kept as it is
e=[1;1] ;
H1x = H1*(Tb-Th) ;   % Transforming the H-matrix
% Set point with composition 
csx = csT-(H1*(e*Th)) ; % Transforming the set-point into composition 

% The new H-matrix for compositon
Htype = [H1x H2];

% Using the self optimizing variable: 
% Optimal setpoint     
           
% Placing the H-elements at teh correct location 
Aeq(1,8)  = Htype(1,1) ;
Aeq(1,22) = Htype(1,2) ;
Aeq(1,23) = Htype(1,3) ;
Aeq(1,26) = Htype(1,4) ;
Aeq(1,30) = Htype(1,5) ;

beq = csx ;   
 
%%

% --------------------- % --------------------- % --------------------- %
n=1 ;

%for n=1:ns
% Change in the Feed - disturbance
d_F0 = (F0_int-(Fdist*F0_int))+(2*(Fdist*F0_int)) - F0_int;      
% .*rand(1,ns)
p.F0 = F0_int + d_F0(:,n) ;


% Using boundry conditions to keep a variable constant
% Feed:
    lb(p.NT+8)  = p.F0;
    ub(p.NT+8)  = p.F0;
    
% fmincon - optimizing for the disturbance with 1 DOF
[xopt(:,n),fvalopt(:,n),exitflag(:,n)]=fmincon(@fun1,x0,[],[],[],[],lb,ub,@nlcon1,options);
costopt(:,n)= xopt(p.NT+2,n)*60 ; 
exit_opt(:,n) = exitflag(:,n) ;    
    
    
% fmincon - no DOF left, process controlled with the self-opt. var.
[xsoc(:,n),fvalsoc(:,n),exitflag(:,n)]=fmincon(@fun1,x0,[],[],Aeq,beq,lb,ub,@nlcon1,options);
costsoc(:,n)= xsoc(p.NT+2,n)*60 ; 
Feed = p.F0 ;

exit_soc(:,n) = exitflag(:,n) ;


av_resi_modelval(:,n) = sum(abs(J_diff))/50
% 
%end 

exitflag = [exit_opt; exit_soc] ;
loss(:,n) = (fvalopt(:,n)-fvalsoc(:,n))*60 ; 
fvalopt;
fvalsoc;
Htype;
if Method == 1
     disp('Hmat')
elseif  Method == 2
     disp('Hexl')
    exlns = (fvalopt(:,n)-fvalsoc(:,n))*60  ;
elseif    Method == 3
     disp('Hns')
    lossns = (fvalopt(:,n)-fvalsoc(:,n))*60  ;
end  