%% EVAPORATOR CASE STUDY : evap10y.m %%
%% 07.06.14 %%
%Test case to calculate H, and comparing it to H found from PLS method
% clear all
% close all
% clc

%% Generate data Ffom case study matrices 
% u = [F200 F1]
% d = [X1 T1 T200]
% Loading data file from generatedata.m
load evapdata.mat

%% Gain matrices  
% Taken from the article 
Gy=[-0.0930 11.678;   % P2
    -0.0520 6.5590;   % T2
    -0.0470 5.9210;   % T3
     0.0000 0.1410;   % F2
    -0.0010 1.1150;   % F100
    -0.0940 2.1700;   % T201
    -0.0320 6.5940;   % F3
     0.0000 0.8590;   % F5
     1.0000 0.0000;   % F200
     0.0000 1.0000];  %F1
 
Gyd=[-3.6260    0    1.9720;    % P2
     -2.0360    0    1.1080;    % T2
     -1.8380    0    1.0000;    % T3
     0.2670     0    0.0000;    % F2
     -0.3170 -0.0180 0.0200;    % F100
     -0.6740    0    1.0000;    % T201
     -2.2530 -0.0660 0.6730;    % F3
     -0.2670    0    0.0000;    % F5
      0         0       0  ;    % F200
      0         0       0  ];   % F1

Juu=[0.0060 -0.1330;
    -0.1330 16.7370];

Jud=[0.0230   0.0000 -0.0010;
    -158.373 -1.1610 1.4830];

Jdd = eye(3,3);
Jsd = [Juu  Jud;
       Jud' Jdd];

% The full gain matrix
Gp=[Gy Gyd];
Jcomb=[Juu' Jud];

% H-matrix according to the null space method 
Hns=Jcomb*pinv(Gp);  % pseudoinverse of Gp

%% PLS test
ny=10;
nu=2;
nd=3;
%ncomp=4;

% Inputs and disturbances 
u = [mF1 ;
     mF200]; 
d = [mX1 ; 
     mT1 ; 
     mT200];
 
ud=[mF1 ;
    mF200; 
    mX1 ; 
    mT1 ; 
    mT200] ;

% Measurements: 
% y = [P2       T2    T3   F2    F100   T201    F3     F5    F200   F1] (10 measurements)
Wn = [1.285     0     0     0      0      0     0      0      0      0;
      0         1     0     0      0      0     0      0      0      0;
      0         0     1     0      0      0     0      0      0      0;
      0         0     0     0.027  0      0     0      0      0      0;
      0         0     0     0      0.189  0     0      0      0      0;
      0         0     0     0      0      1     0      0      0      0;
      0         0     0     0      0      0     0.494  0      0      0;
      0         0     0     0      0      0     0      0.163  0      0;
      0         0     0     0      0      0     0      0      4.355  0;
      0         0     0     0      0      0     0      0      0      0.189];
a=kron(diag(Wn),ones(1,ns));
noise = (a-(0.1*a))+(2*(0.1*a).*rand(ny,ns));
Ally = (Gp*ud) + noise;         % With measurement noise
%Ally = (Gp*ud) ;               % Without measurement noise


%% Center the data
% each element in a row minus the mean of the row
% Ym=mean(Ally,2)
% Ym=kron(Ym,ones(1,ns))
% Y=Ally-Ym                % Ally with centered data 

% Scaling the values
% The measurements
Ymax = max(abs(Ally'))' ;
Yscal=[];
for i=1:ny
    Yscal(i,:) = Ally(i,:)/Ymax(i,1);
end
    
scy = diag(Ymax) ;

% Finding Yaug:
Yadd=[] ;
l=1 ;
for i =1:size(Ally,1)
    for j =i:size(Ally,1)
        Yadd(l,:) = Yscal(i,:).*Yscal(j,:) ;
        l=l+1 ;
    end
end

Yaug=[Yscal; Yadd];

% The cost function
Jlin = ud'*Jsd*ud;

Jorg=diag(Jlin);
J=Jorg-repmat(mean(Jorg),ns,1); %centered

Jmax=max(abs(J));
Jscal=J/Jmax;


 %%

[P,Q,T,U,beta,pctvar] = plsregress(Yaug',Jscal,ncomp);
% X-scores T
% X-loadings W
% Y-scores U
% Y loading C
beta=beta' ;
 

Test=[ones(ns,1) Yaug']*beta';
% 
% Making Jyy:
a=0 ;
for i=1:ny
    for j=i:ny
        Jy(i,j)=beta(ny+2+a);
        a=a+1;
    end 
end
Jy=Jy;
Jyy=Jy+Jy';


%% Checking if the H are the same
Hmat=[Gy';zeros(nd,ny)]*Jyy ; %/scy ;
Hmat=[Gy';zeros(nd,ny)]*Jyy/scy; 
Hmat = Hmat(1:nu,:)              ;   % This is the "first H-matrix we found before any scaling
Hmatsc = Hmat(1:nu,1:nu)\Hmat     ;  % Scaled H matrix 
H = Hns(1:nu,1:nu)\Hns   ;               % H from Jcomp*Gp    
%Haux =[Gy'; zeros(2,4)]*Jyy/scY(1:4,1:4);
 
res = H-Hmatsc ;

% figure(1)
% plot(1:ncomp,cumsum(100*pctvar(2,:)),'-bo');
% xlabel('number of PLS components')
% ylabel('Precent Varance Explained in J')


%% Exact local method 
Wd = [0.25 0 0;
        0 8 0;
        0 0 5];
     
Wn = [1.285     0     0     0      0      0     0      0      0      0;
      0         1     0     0      0      0     0      0      0      0;
      0         0     1     0      0      0     0      0      0      0;
      0         0     0     0.027  0      0     0      0      0      0;
      0         0     0     0      0.189  0     0      0      0      0;
      0         0     0     0      0      1     0      0      0      0;
      0         0     0     0      0      0     0.494  0      0      0;
      0         0     0     0      0      0     0      0.163  0      0;
      0         0     0     0      0      0     0      0      4.355  0;
      0         0     0     0      0      0     0      0      0      0.189];

% % Sensitivity matrix 
F= Gyd-(Gy*Juu^(-1)*Jud) ;
Y = [F*Wd Wn];
Hexl=Gy'*(Y*Y')^(-1);


  Y = [F*Wd Wn];              % With measurement noise
 %Y = [F*Wd Wn*0];            % Without measurement noise

Mexl = Juu^(0.5) *(Hexl*Gy)^(-1) * (Hexl*Y);
Loss_exl = 0.5*norm(Mexl,'fro').^2

Mdata = Juu^(0.5) *(Hmat*Gy)^(-1) * (Hmat*Y);
Loss_data = 0.5*norm(Mdata,'fro').^2

Mns = Juu^(0.5) *(Hns*Gy)^(-1) * (Hns*Y);
Loss_ns = 0.5*norm(Mns,'fro').^2


Hns1 = null(F')';
Hns1 = Hns(1:2,:);
Mns1 = Juu^(0.5) *(Hns1*Gy)^(-1) * (Hns1*Y);
Loss_ns1 = 0.5*norm(Mns1,'fro').^2

Hmat
Hexl
Hns




