%% EVAPORATOR CASE STUDY : evapgeneratedata.m %%
%% 07.06.14 %%

clear all 
close all
clc
%% Set how many data sample set you want, ns, 
ns=1000;
nsv = ones(1,ns);
r=-1+2*rand(1,ns);      % random nr b/w -1 and 1
%% Generating data - The evaporator process: 
%The measurement y=Gy*u + Gyd*d = Gp*[u d]'
% y = [P2   T2  T3  F2  F100  T201  F3  F5  F200   F1] (10 measurements)

%% Initial values
% Inputs: 
F2000 = 208.0;        % Cooling water flowrate kg/min
F10    = 10.0;         % Feed flowrate kg/min

% Disturbances: 
X10   = 5;              % Feed composition percent %
T10   = 40;             % Feed temperature C
T2000 = 25;             % Coolingwater inlet temperature C

% Initial values
ud0=[F2000;
     F10;
     X10;
     T10;
     T2000];

  
  
%% Initial values given in book
P20   = 50.5;         % Operating pressure                  kPa
T20   = 84.6;         % Product temperature                 C
T30   = 80.6;         % Vapor temperature                   C
F20   = 2;            % Product flowrate                    kg/min
F1000 = 9.3;          % Stream flowrate                     kg/min
T2010 = 46.1;         % Cooling water outlet temperature    C
F30   = 50.0;         % Circulating flowrate                kg/min
F50   = 8.0;          % Condencate flowrate                 kg/min
F2000 = 208.0;        % Cooling water flowrate              kg/min
F10    = 10.0;         % Feed flowrate                      kg/min

y0=[P20   ;
    T20   ;
    T30   ;
    F20   ;
    F1000 ;
    T2010 ;
    F30   ;
    F50   ;
    F2000 ;
    F10   ] ;

%% Generate random data
per=20  ;              
perd=5 ;
% Inputs - in deviation variables  
%mF200 =  (F2000-(0.1*F2000))+(2*(0.1*F2000)*rand(1,ns))-F2000 ;      % +- 10%
mF200 =  -0.1*F2000+0.2*F2000*rand(1,ns)   ;  % +- 10%

mF1   =  (F10-(0.1*F10))+(2*(0.1*F10).*rand(1,ns))-F10;               % +- 10%

% Disturbances - in deviation variables 
mX1   = (X10-(0.05*X10))+(2*(0.05*X10)*rand(1,ns))-X10;               % +- 5%  
mT1   = (T10-(0.2*T10))+(2*(0.2*T10)*rand(1,ns))-T10;                 % +- 20%  
mT200 = (T2000-(0.2*T2000))+(2*(0.2*T2000)*rand(1,ns))-T2000;         % +- 20%  

% [int-(10% av int)] + 2*(10% av int)*rand(1,ns) - int 

%y = [mP2 ; mT2 ; mT3 ; mF2 ; mF100 ; mT201 ; mF100 ; mF3 ; mF5 ; mF200 ; mF1];

% optimal J
Jnom = 600*F1000 + 0.6*F2000 + 1.009*(F20 + F30) + 0.2*F10 - 4800*F20;


save('evapdata','mF200', 'mF1', 'mX1' ,'mT1', 'mT200','Jnom','ns','y0')
